import sys

# Project libraries
from . import BaseResource

if sys.version_info > (3, 0):
    from urllib.parse import quote
else:
    from urllib import quote  # noqa


class Formation(BaseResource):

    _strs = ["id", "command", "type", "size"]
    _ints = ["quantity"]
    _bools = ["attached"]
    _dates = ["created_at", "updated_at"]
    _pks = ["id", "type"]

    def __init__(self):
        self.app = None
        super(Formation, self).__init__()

    def __repr__(self):
        return "<formation '{0} - {1}'>".format(self.type, self.command)

    def scale(self, quantity):
        """Scales the given process to the given number of dynos."""
        return self.update(quantity=quantity)

    def resize(self, size):
        """Resizes the Dynos to the multiple given"""
        return self.update(size=size)

    def update(self, size=None, quantity=None):

        assert size or quantity == 0 or quantity
        payload = {}
        if size:
            payload["size"] = size

        if quantity or quantity == 0:
            payload["quantity"] = quantity

        r = self._h._http_resource(
            method="PATCH",
            resource=("apps", self.app.id, "formation", quote(self.type)),
            data=self._h._resource_serialize(payload),
        )

        r.raise_for_status()
        return self._h._process_item(self._h._resource_deserialize(r.content.decode("utf-8")), Formation)



# Project libraries
from . import BaseResource
from .app import App
from .build import Build


class PostDeploy(BaseResource):
    _strs = ["output"]
    _ints = ["exit_code"]

    def __init__(self):
        super(PostDeploy, self).__init__()


class AppSetup(BaseResource):
    _strs = ["id", "failure_message", "postdeploy:exit_code", "postdeploy:output", "resolved_success_url", "status"]
    _map = {"app": App, "build": Build, "postdeploy": PostDeploy}
    _dates = ["created_at", "updated_at"]
    _pks = ["id"]

    def __init__(self):
        super(AppSetup, self).__init__()

    def __repr__(self):
        return "<appsetup '{0}' - '{1}'>".format(self.id, self.status)



# Project libraries
from . import BaseResource
from .release import Release


class RestartRunException(Exception):
    pass


class Dyno(BaseResource):
    _strs = ["id", "attach_url", "size", "command", "name", "state", "type"]
    _bools = ["attach"]
    _ints = ["repo_size"]
    _dates = ["created_at", "updated_at"]
    _map = {"release": Release}
    _pks = ["id"]

    def __init__(self):
        self.app = None
        super(Dyno, self).__init__()

    def __repr__(self):
        return "<Dyno '{0} - {1}'>".format(self.name, self.command)

    def kill(self):
        r = self._h._http_resource(method="DELETE", resource=("apps", self.app.id, "dynos", self.name))

        r.raise_for_status()
        
        return r.ok
    
    def stop_dyno(self):
        data = {'ps': self.process}
        
        r = self._h._http_resource(method='POST', resource=('apps', self.app.name, 'ps', 'stop'))
        r.raise_for_status()
        
    def restart(self):
        if self.type == "run":
            raise RestartRunException(
                "Unable to restart a Process of type 'run' as it will not be respawned by Heroku"
            )
        else:
            return self.kill()



# Project libraries
from . import User, BaseResource


class Collaborator(BaseResource):
    """Heroku Collaborator."""

    _bools = ["silent"]
    _dates = ["created_at", "updated_at"]
    _strs = ["id"]
    _pks = ["id"]
    _map = {"user": User}

    def __init__(self):
        self.app = None
        super(Collaborator, self).__init__()

    def __repr__(self):
        return "<collaborator '{0}'>".format(self.user.email)

    def remove(self):
        r = self._h._http_resource(method="DELETE", resource=("apps", self.app.name, "collaborators", self.user.email))
        r.raise_for_status()

        return r.ok
